@@ -2,6 +2,7 @@
 """Python Package for auth capture proxy."""
 import asyncio
 import logging
+import re
 from functools import partial
 from ssl import SSLContext
 from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union
@@ -10,6 +11,7 @@ import multidict
 from aiohttp import (
     ClientConnectionError,
     ClientSession,
+    ClientTimeout,
     MultipartReader,
     MultipartWriter,
     TooManyRedirects,
@@ -19,6 +21,7 @@ from aiohttp import (
 from aiohttp.client_reqrep import ClientResponse
 from yarl import URL
 
+from authcaptureproxy.const import SKIP_AUTO_HEADERS
 from authcaptureproxy.examples.modifiers import (
     prepend_relative_urls,
     replace_empty_action_urls,
@@ -47,7 +50,9 @@ class AuthCaptureProxy:
             session (ClientSession): Session to make aiohttp queries. Optional
 
         """
-        self.session: ClientSession = session if session else ClientSession()
+        self.session: ClientSession = (
+            session if session else ClientSession(timeout=ClientTimeout(total=30))
+        )
         self._proxy_url: URL = proxy_url
         self._host_url: URL = host_url
         self._port: int = proxy_url.explicit_port if proxy_url.explicit_port else 0  # type: ignore
@@ -70,6 +75,9 @@ class AuthCaptureProxy:
         self._active = False
         self._all_handler_active = True
         self.headers: Dict[Text, Text] = {}
+        self.redirect_filters: Dict[Text, List[Text]] = {
+            "url": []
+        }  # dictionary of lists of regex strings to filter against
 
     @property
     def active(self) -> bool:
@@ -273,21 +281,26 @@ class AuthCaptureProxy:
         if not self.session:
             self.session = ClientSession()
         method = request.method.lower()
-        _LOGGER.debug("Received %s: %s", method, request.url)
+        _LOGGER.debug("Received %s: %s for %s", method, request.url, self._host_url)
         resp: Optional[ClientResponse] = None
+        old_url: URL = (
+            self.access_url().with_host(request.url.host)
+            if request.url.host and request.url.host != self.access_url().host
+            else self.access_url()
+        )
         if request.scheme == "http" and self.access_url().scheme == "https":
             # detect reverse proxy downgrade
             _LOGGER.debug("Detected http while should be https; switching to https")
             site: URL = swap_url(
                 ignore_query=True,
-                old_url=self.access_url(),
+                old_url=old_url,
                 new_url=self._host_url.with_path("/"),
                 url=request.url.with_scheme("https"),
             )
         else:
             site = swap_url(
                 ignore_query=True,
-                old_url=self.access_url(),
+                old_url=old_url,
                 new_url=self._host_url.with_path("/"),
                 url=request.url,
             )
@@ -334,8 +347,12 @@ class AuthCaptureProxy:
                         self._host_url,
                     )
             headers = await self.modify_headers(site, request)
+            skip_auto_headers: List[str] = headers.get(SKIP_AUTO_HEADERS, [])
+            if skip_auto_headers:
+                _LOGGER.debug("Discovered skip_auto_headers %s", skip_auto_headers)
+                headers.pop(SKIP_AUTO_HEADERS)
             _LOGGER.debug(
-                "Attempting %s to %s: \nheaders:%s \ncookies: %s",
+                "Attempting %s to %s\nheaders: %s \ncookies: %s",
                 method,
                 site,
                 headers,
@@ -344,15 +361,11 @@ class AuthCaptureProxy:
             try:
                 if mpwriter:
                     resp = await getattr(self.session, method)(
-                        site,
-                        data=mpwriter,
-                        headers=headers,
+                        site, data=mpwriter, headers=headers, skip_auto_headers=skip_auto_headers
                     )
                 elif data:
                     resp = await getattr(self.session, method)(
-                        site,
-                        data=data,
-                        headers=headers,
+                        site, data=data, headers=headers, skip_auto_headers=skip_auto_headers
                     )
                 elif json_data:
                     for item in ["Host", "Origin", "User-Agent", "dnt", "Accept-Encoding"]:
@@ -363,12 +376,11 @@ class AuthCaptureProxy:
                         site,
                         json=json_data,
                         headers=headers,
-                        # apparently headers aren't needed for Tesla case?
+                        skip_auto_headers=skip_auto_headers,
                     )
                 else:
                     resp = await getattr(self.session, method)(
-                        site,
-                        headers=headers,
+                        site, headers=headers, skip_auto_headers=skip_auto_headers
                     )
             except ClientConnectionError as ex:
                 return web.Response(text=f"Error connecting to {site}; please retry: {ex}")
@@ -539,6 +551,11 @@ class AuthCaptureProxy:
     async def modify_headers(self, site: URL, request: web.Request) -> multidict.MultiDict:
         """Modify headers.
 
+        Return modified headers based on site and request. To disable auto header generation,
+        pass in a key const.SKIP_AUTO_HEADERS with a list of keys to not generate.
+
+        For example, to prevent User-Agent generation: {SKIP_AUTO_HEADERS : ["User-Agent"]}
+
         Args:
             site (URL): URL of the next host request.
             request (web.Request): Proxy directed request. This will need to be changed for the actual host request.
@@ -572,7 +589,10 @@ class AuthCaptureProxy:
         return multidict.MultiDict(result)
 
     def check_redirects(self) -> None:
-        """Change host if redirect detected."""
+        """Change host if redirect detected and regex does not match self.redirect_filters.
+
+        Self.redirect_filters is a dict with key as attr in resp and value as list of regex expressions to filter against.
+        """
         if not self.last_resp:
             return
         resp: ClientResponse = self.last_resp
@@ -582,6 +602,32 @@ class AuthCaptureProxy:
                     item.status in [301, 302, 303, 304, 305, 306, 307, 308]
                     and resp.url.host != self._host_url.host
                 ):
+                    filtered = False
+                    for attr, regex_list in self.redirect_filters.items():
+                        if getattr(resp, attr) and list(
+                            filter(
+                                lambda regex_string: re.search(
+                                    regex_string, str(getattr(resp, attr))
+                                ),
+                                regex_list,
+                            )
+                        ):
+                            _LOGGER.debug(
+                                "Check_redirects: Filtered out on %s in %s for resp attribute %s",
+                                list(
+                                    filter(
+                                        lambda regex_string: re.search(
+                                            regex_string, str(getattr(resp, attr))
+                                        ),
+                                        regex_list,
+                                    )
+                                ),
+                                str(getattr(resp, attr)),
+                                attr,
+                            )
+                            filtered = True
+                    if filtered:
+                        return
                     _LOGGER.debug(
                         "Detected %s redirect from %s to %s; changing proxy host",
                         item.status,
